﻿using System;
using HAServer.Components;
using HAServer.Interfaces.Core;
using Moq;
using NUnit.Framework;

namespace HAServer.Tests.Components {
	[TestFixture]
	public class SolarEventTests {
		private const string Json = @"{""results"":{""sunrise"":""2017-05-29T03:49:06+00:00"",""sunset"":""2017-05-29T20:22:45+00:00"",""solar_noon"":""2017-05-29T12:05:55+00:00"",""day_length"":59619,""civil_twilight_begin"":""2017-05-29T03:00:26+00:00"",""civil_twilight_end"":""2017-05-29T21:11:25+00:00"",""nautical_twilight_begin"":""2017-05-29T01:43:51+00:00"",""nautical_twilight_end"":""2017-05-29T22:28:00+00:00"",""astronomical_twilight_begin"":""1970-01-01T00:00:01+00:00"",""astronomical_twilight_end"":""1970-01-01T00:00:01+00:00""},""status"":""OK""}";

		[Test]
		public void Constructor_GetsDaysEvents() {
			var bus = new Mock<IMessageBus>();
			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");
			var mockApi = new Mock<IFileGet>();
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Returns(Json);
			var logProvider = new Mock<ILogProvider>();
			var solarEvents = new SolarEvents(bus.Object, settingsProvider.Object, mockApi.Object, logProvider.Object, TimeZoneInfo.Utc);
			solarEvents.Initialise();
			Assert.That(solarEvents.Events["sunrise"], Is.EqualTo("03:49:06"));
			Assert.That(solarEvents.Events["sunset"], Is.EqualTo("20:22:45"));
			Assert.That(solarEvents.Events["solar_noon"], Is.EqualTo("12:05:55"));
			Assert.That(solarEvents.Events["civil_twilight_begin"], Is.EqualTo("03:00:26"));
			Assert.That(solarEvents.Events["civil_twilight_end"], Is.EqualTo("21:11:25"));
			Assert.That(solarEvents.Events["nautical_twilight_begin"], Is.EqualTo("01:43:51"));
			Assert.That(solarEvents.Events["nautical_twilight_end"], Is.EqualTo("22:28:00"));
			Assert.That(solarEvents.Events["astronomical_twilight_begin"], Is.EqualTo("00:00:01"));
			Assert.That(solarEvents.Events["astronomical_twilight_end"], Is.EqualTo("00:00:01"));
		}

		[Test]
		public void ProcessMessage_WithMatchingTime_PublishesEvent() {
			var callbackMessage = string.Empty;

			var bus = new Mock<IMessageBus>();
			bus.Setup(messageBus => messageBus.PublishMessage(It.IsAny<string>(), It.IsAny<string>(), It.IsAny<ISubscriber>()))
			   .Callback((string topic, string message, ISubscriber subscriber) => { callbackMessage = message; });

			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");

			var mockApi = new Mock<IFileGet>();
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Returns(Json);

			var logProvider = new Mock<ILogProvider>();

			var solarEvents = new SolarEvents(bus.Object, settingsProvider.Object, mockApi.Object, logProvider.Object, TimeZoneInfo.Utc);
			solarEvents.Initialise();
			solarEvents.ProcessMessage("time", "21:11:25");

			Assert.That(callbackMessage, Is.EqualTo("civil_twilight_end"));
		}

		[Test]
		public void AtMidnight_RefreshSolarEvents() {
			var bus = new Mock<IMessageBus>();
			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");
			var mockApi = new Mock<IFileGet>();
			var logProvider = new Mock<ILogProvider>();
			var solarEvents = new SolarEvents(bus.Object, settingsProvider.Object, mockApi.Object, logProvider.Object, TimeZoneInfo.Utc);
			var updateCalled = false;
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Callback(() => { updateCalled = true; });
			solarEvents.ProcessMessage("time", "00:00:00");
			Assert.That(updateCalled, Is.True);
		}

		[Test]
		public void GetSolarEventsForTheDay_WithError_WritesErrorToConsole() {
			var bus = new Mock<IMessageBus>();
			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");
			var mockApi = new Mock<IFileGet>();
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Returns(@"{""results"":"""",""status"":""UNKNOWN_ERROR""}");
			var logProvider = new Mock<ILogProvider>();
			var output=string.Empty;
			logProvider.Setup(provider => provider.Write(It.IsAny<string>(), It.IsAny<LogLevel>()))
					   .Callback((string message, LogLevel level) => { output = message; });
			var solarEvents = new SolarEvents(bus.Object, settingsProvider.Object, mockApi.Object, logProvider.Object, TimeZoneInfo.Utc);
			solarEvents.Initialise();
			Assert.That(output, Is.EqualTo("FAILED TO GET SOLAR EVENTS!"));
		}

		[Test]
		public void GetSolarEventsForTheDay_WithTimezone_AdjustsForTimezone() {
			var bus = new Mock<IMessageBus>();
			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");
			var mockApi = new Mock<IFileGet>();
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Returns(Json);
			var logProvider = new Mock<ILogProvider>();
			var customTimeZone = TimeZoneInfo.CreateCustomTimeZone("id", TimeSpan.FromHours(1), "test", "test");
			var solarEvents = new SolarEvents(bus.Object, settingsProvider.Object, mockApi.Object, logProvider.Object, customTimeZone);
			solarEvents.Initialise();
			Assert.That(solarEvents.Events["sunrise"], Is.EqualTo("04:49:06"));
			Assert.That(solarEvents.Events["sunset"], Is.EqualTo("21:22:45"));
			Assert.That(solarEvents.Events["solar_noon"], Is.EqualTo("13:05:55"));
			Assert.That(solarEvents.Events["civil_twilight_begin"], Is.EqualTo("04:00:26"));
			Assert.That(solarEvents.Events["civil_twilight_end"], Is.EqualTo("22:11:25"));
			Assert.That(solarEvents.Events["nautical_twilight_begin"], Is.EqualTo("02:43:51"));
			Assert.That(solarEvents.Events["nautical_twilight_end"], Is.EqualTo("23:28:00"));
			Assert.That(solarEvents.Events["astronomical_twilight_begin"], Is.EqualTo("01:00:01"));
			Assert.That(solarEvents.Events["astronomical_twilight_end"], Is.EqualTo("01:00:01"));
		}
	}
}
