﻿using System;
using System.Collections.Generic;
using System.IO;
using HAServer.Interfaces.Core;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace HAServer.Components {
	public class SolarEvents:ISubscriber, IComponent {
		private readonly IMessageBus _messageBus;
		private readonly ISettingsProvider _settingsProvider;
		private readonly IFileGet _fileGetter;
		private readonly ILogProvider _logProvider;
		private readonly TimeZoneInfo _timeZone;
		private Dictionary<string, string> _events = new Dictionary<string, string>();
		private readonly JsonSerializer _serialiser;

		public SolarEvents(IMessageBus messageBus, ISettingsProvider settingsProvider, IFileGet fileGetter, ILogProvider logProvider, TimeZoneInfo timeZone) {
			_messageBus = messageBus;
			_settingsProvider = settingsProvider;
			_fileGetter = fileGetter;
			_logProvider = logProvider;
			_timeZone = timeZone;

			_serialiser = new JsonSerializer {
				DateParseHandling = DateParseHandling.None
			};
		}

		private bool GetSolarData(out JObject events) {
			events = null;

			var json = _fileGetter.Get($"http://api.sunrise-sunset.org/json?lat={_settingsProvider.Latitude()}&lng={_settingsProvider.Longitude()}&formatted=0");
			if (string.IsNullOrWhiteSpace(json))
				return false;

			JsonReader reader = new JsonTextReader(new StringReader(json));
			var result = _serialiser.Deserialize<JObject>(reader);

			if (result["status"].ToString().ToLower() != "ok") {
				_logProvider.Write("FAILED TO GET SOLAR EVENTS!", LogLevel.Warning);
				return false;
			}

			events = result["results"].ToObject<JObject>();
			return true;
		}

		private bool ParseDateForTimeZone(string dateTime, out DateTime time) {
			time = new DateTime();
			if (!DateTimeOffset.TryParse(dateTime, out var parsedDate))
				return false;
			var utc = DateTime.SpecifyKind(parsedDate.DateTime, DateTimeKind.Utc);
			time = TimeZoneInfo.ConvertTimeFromUtc(utc, _timeZone);
			return true;
		}

		private void ParseEvents(JObject events) {
			_events = new Dictionary<string, string>();
			foreach (var @event in events) {
				var timeString = @event.Value.ToString();
				if (ParseDateForTimeZone(timeString, out var time))
					_events[@event.Key] = time.ToTimeString();
			}
		}

		private void GetSolarEventsForTheDay() {
			if (GetSolarData(out var events))
				ParseEvents(events);
		}

		public void ProcessMessage(string topic, string message) {
			if (message == "00:00:00")
				GetSolarEventsForTheDay();

			foreach (var @event in _events) {
				if (@event.Value==message)
					_messageBus.PublishMessage("solar_event", @event.Key, this);
			}
		}

		public Dictionary<string, string> Events => _events;

		public void Initialise() {
			_messageBus.AddSubscriber(this, "^time$");
			GetSolarEventsForTheDay();
		}

		public void Shutdown() {
			
		}
	}
}
