﻿using System;
using FluentAssertions;
using HAServer.Core;
using Xunit;

namespace HAServer.Tests.Core {
	public class DependenciesTests {
		private interface IDummy { }
		private interface IDummy2 { }
		private class Dummy:IDummy { }
		private class Dummy2:IDummy2 {
			public Dummy2(IDummy dummy) {}
		}
		private class Dummy3:IDummy {
			public readonly bool PickedCtor1;
			public readonly bool PickedCtor2;
			public Dummy3(IDummy dummy) {
				PickedCtor1 = true;
			}
			public Dummy3(string test) {}
			public Dummy3(IDummy dummy, IDummy2 dummy2) {
				PickedCtor2 = true;
			}
			public Dummy3() {}
		}
		private class Dummy4 {
			public Dummy4(string test) {}
		}

		[Fact]
		public void Register_Resolve() {
			var dependencies = new DependencyResolver();
			var dummy = new Dummy();
			dependencies.Register<IDummy>(dummy);
			Assert.Equal(dummy, dependencies.Resolve<IDummy>());
		}

		[Fact]
		public void Register_WithExistingType_ThrowsException() {
			var dependencies = new DependencyResolver();
			var dummy = new Dummy();
			dependencies.Register<IDummy>(dummy);
			Action action = () => { dependencies.Register<IDummy>(dummy); };
			action.Should().Throw<Exception>();
		}

		[Fact]
		public void Resolve_WithNotExistingType_ThrowsException() {
			var dependencies = new DependencyResolver();
			Action action = () => { dependencies.Resolve<IDummy>(); };
			action.Should().Throw<Exception>();
		}

		[Fact]
		public void CreateIntance_WithType_ReturnsType() {
			var dependencies=new DependencyResolver();
			var obj = dependencies.CreateInstance(typeof(Dummy));
			obj.Should().BeOfType<Dummy>();
		}

		[Fact]
		public void CreateIntance_WithTypeWithoutParameterlessConstructor_ReturnsType() {
			var dependencies = new DependencyResolver();
			var dummy = new Dummy();
			dependencies.Register<IDummy>(dummy);
			var obj = dependencies.CreateInstance(typeof(Dummy2));
			obj.Should().BeOfType<Dummy2>();
		}

		[Fact]
		public void CreateIntance_WithTypeWithManyConstructors_ReturnsType() {
			var dependencies = new DependencyResolver();
			var dummy = new Dummy();
			var dummy2 = new Dummy2(dummy);
			dependencies.Register<IDummy>(dummy);
			dependencies.Register<IDummy2>(dummy2);
			var obj = dependencies.CreateInstance<Dummy3>();
			obj.PickedCtor2.Should().BeTrue();
		}

		[Fact]
		public void CreateIntance_WithTypeWithManyConstructors_SelectsBestMatch() {
			var dependencies = new DependencyResolver();
			var dummy = new Dummy();
			var dummy2 = new Dummy2(dummy);
			dependencies.Register<IDummy>(dummy);
			var obj = dependencies.CreateInstance<Dummy3>();
			obj.PickedCtor1.Should().BeTrue();
		}

		[Fact]
		public void CreateIntance_WhenUnableToFindTypeInConstructor_ThrowsException() {
			var dependencies = new DependencyResolver();
			Action action = ()=>{ dependencies.CreateInstance<Dummy4>(); };
			action.Should().Throw<Exception>();
		}
	}
}
