﻿using System;
using System.IO;
using System.Linq;
using FluentAssertions;
using HAServer.Core;
using HAServer.Interfaces.Core;
using HAServer.Tests.Stubs;
using NSubstitute;
using Xunit;

namespace HAServer.Tests.Core {
	public class DeviceConfigTests {
		private const string ExpectedJson = @"{
  ""Devices"": {
    ""topic"": {
      ""Type"": ""Sensor"",
      ""Value"": null,
      ""Unit"": """",
      ""Topic"": ""topic"",
      ""Name"": ""name""
    }
  },
  ""Groups"": {
    ""test"": [
      ""topic""
    ]
  }
}";

		[Fact]
		public void AddDevice_WithDevice_AddsADevice() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			deviceConfig.Devices.Should().ContainValue(device);
		}

		[Fact]
		public void RemoveDevice_WithDeviceTopic_RemovesDevice() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			deviceConfig.RemoveDevice(device.Topic);
			deviceConfig.Devices.Should().NotContainValue(device);
		}

		[Fact]
		public void RemoveDevice_WithDeviceTopic_RemovesDeviceFromGroups() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			deviceConfig.AddGroup("test");
			deviceConfig.AddDeviceToGroup("test", device.Topic);
			deviceConfig.RemoveDevice(device.Topic);
			deviceConfig.Groups["test"].Should().NotContain(device);
		}

		[Fact]
		public void AddGroup_WithGroupName_AddsGroup() {
			var deviceConfig = new DeviceConfig();
			deviceConfig.AddGroup("test");
			deviceConfig.Groups.FirstOrDefault(group => group.Name == "test").Should().NotBeNull();
		}

		[Fact]
		public void RemoveGroup_WithGroupName_RemovesGroup() {
			var deviceConfig = new DeviceConfig();
			deviceConfig.AddGroup("test");
			deviceConfig.RemoveGroup("test");
			deviceConfig.Groups.FirstOrDefault(group => group.Name == "test").Should().BeNull();
		}

		[Fact]
		public void AddDeviceToGroup_WithValidGroupNameAndValidDeviceTopic_AddsDeviceToGroup() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			deviceConfig.AddGroup("test");
			deviceConfig.AddDeviceToGroup("test", device.Topic);
			deviceConfig.Groups["test"].Should().Contain(device);
		}

		[Fact]
		public void AddDeviceToGroup_WithValidGroupNameAndInvalidDeviceTopic_ThrowsException() {
			var deviceConfig = new DeviceConfig();
			deviceConfig.AddGroup("test");
			Action action = () => deviceConfig.AddDeviceToGroup("test", "device");
			action.Should().Throw<ArgumentException>();
		}

		[Fact]
		public void AddDeviceToGroup_WithInvalidGroupNameAndValidDeviceTopic_AddsDeviceToGroup() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			Action action = () => deviceConfig.AddDeviceToGroup("test", device.Topic);
			action.Should().Throw<ArgumentException>();
		}

		[Fact]
		public void RemoveDeviceFromGroup_WithGroupNameAndDeviceTopic_DeviceIsRemovedFromGroup() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			deviceConfig.AddGroup("test");
			deviceConfig.AddDeviceToGroup("test", device.Topic);
			deviceConfig.RemoveDeviceFromGroup("test", device.Topic);
			deviceConfig.Groups["test"].Should().NotContain(device.Topic);
		}

		[Fact(Skip = "Too fragile")]
		public void Save_WithFilename_SavesConfigToJson() {
			var deviceConfig = new DeviceConfig();
			var bus = Substitute.For<IMessageBus>();
			var device = new DeviceStub(bus, "topic", "name");
			deviceConfig.AddDevice(device);
			deviceConfig.AddGroup("test");
			deviceConfig.AddDeviceToGroup("test", device.Topic);
			var testJsonPath = Path.Combine(Useful.GetTestDirectory(), "test.json");
			deviceConfig.Save(testJsonPath);
			File.ReadAllText(testJsonPath).Should().BeEquivalentTo(ExpectedJson);
			File.Delete(testJsonPath);
		}

		[Fact]
		public void Load_ReturnsValidDeviceConfig() {
			var testJsonPath = Path.Combine(Useful.GetTestDirectory(), "test.json");
			File.WriteAllText(testJsonPath, ExpectedJson);
			var bus = Substitute.For<IMessageBus>();
			var loadedDeviceConfig = DeviceConfig.Load(testJsonPath, bus);
			var device = new DeviceStub(bus, "topic", "name");
			loadedDeviceConfig.Groups.FirstOrDefault(group => group.Name == "test").Should().NotBeNull();
			loadedDeviceConfig.Groups["test"].FirstOrDefault(baseDevice => baseDevice.Topic == device.Topic).Should().NotBeNull();
			loadedDeviceConfig.Devices.Should().ContainKey(device.Topic);
			File.Delete(testJsonPath);
		}
	}
}
