﻿using FluentAssertions;
using HAServer.Core;
using HAServer.Interfaces.Core;
using NSubstitute;
using Xunit;

namespace HAServer.Tests.Core {
	public class MessageBusTests {
		[Fact]
		public void AddSubscriber_AddsSubscriber() {
			var log=Substitute.For<ILogProvider>();
			var bus=new MessageBus(log);
			var subscriber=Substitute.For<ISubscriber>();
			bus.AddSubscriber(subscriber, "topic");
			bus.GetSubscribers("topic").Should().Contain(subscriber);
		}

		[Fact]
		public void RemoveSubscriber_SubscriberIsRemovedFromAllTopics() {
			const string topic1 = "topic1";
			const string topic2 = "topic2";

			var log = Substitute.For<ILogProvider>();
			var bus = new MessageBus(log);
			var subscriber = Substitute.For<ISubscriber>();

			bus.AddSubscriber(subscriber, topic1);
			bus.AddSubscriber(subscriber, topic2);

			bus.RemoveSubscriber(subscriber);

			bus.GetSubscribers(topic1).Should().NotContain(subscriber);
			bus.GetSubscribers(topic2).Should().NotContain(subscriber);
		}

		[Fact]
		public void GetSubscribers_WithRegex_ReturnsSubscribers() {
			var log = Substitute.For<ILogProvider>();
			var bus = new MessageBus(log);
			var subscriber=Substitute.For<ISubscriber>();

			bus.AddSubscriber(subscriber, "/test/.*");

			bus.GetSubscribers("/test/topic").Should().Contain(subscriber);
		}

		[Fact]
		public void PublishMessage_SenderDoesNotGetMessage() {
			var log = Substitute.For<ILogProvider>();
			var bus = new MessageBus(log);
			var subscriber = Substitute.For<ISubscriber>();
			var called = false;

			subscriber.When(sub => sub.ProcessMessage(Arg.Any<string>(), Arg.Any<string>())).Do(callInfo => { called = true; });
			
			bus.AddSubscriber(subscriber, "topic");
			bus.PublishMessage("topic", "test", subscriber);

			called.Should().BeFalse();
		}

		[Fact]
		public void PublishMessage_SubscribersGetMessage() {
			var log = Substitute.For<ILogProvider>();
			var bus = new MessageBus(log);
			var subscriber1 = Substitute.For<ISubscriber>();
			var subscriber2 = Substitute.For<ISubscriber>();
			var callbackMessage = string.Empty;

			subscriber2.When(sub => sub.ProcessMessage(Arg.Any<string>(), Arg.Any<string>())).Do(callInfo => { callbackMessage = callInfo.ArgAt<string>(1); });
			
			bus.AddSubscriber(subscriber2, "topic");
			bus.PublishMessage("topic", "test",subscriber1);

			Assert.Equal("test", callbackMessage);
		}
	}
}
