﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading;
using FluentAssertions;
using HAServer.Core;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using HAServer.Tests.Stubs;
using NSubstitute;
using Xunit;

namespace HAServer.Tests.Core {
	public class TriggerConfigTests {
		private const string ExpectedJson= @"{
  ""Triggers"": [
    {
      ""Id"": 0,
      ""Name"": ""name"",
      ""TriggerType"": 0,
      ""Enabled"": false,
      ""Conditions"": [
        {
          ""Comparator"": 0,
          ""Topic"": ""event"",
          ""Value"": ""value""
        },
        {
          ""Device"": ""topic"",
          ""Comparator"": 0,
          ""Topic"": ""event""
        }
      ],
      ""Actions"": [
        {
          ""Type"": 0,
          ""Identifier"": ""topic"",
          ""Method"": ""Method""
        },
        {
          ""Type"": 1,
          ""Identifier"": ""group"",
          ""Method"": ""TurnOn""
        }
      ]
    }
  ]
}";

		[Fact]
		public void Add_WithTrigger_AddsTrigger() {
			var config = new TriggerConfig();
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();
			var trigger = new Trigger(bus, string.Empty, TriggerType.All, Enumerable.Empty<TriggerAction>(), Enumerable.Empty<Condition>(), logProvider);
			config.Add(trigger);
			config.Triggers.Should().Contain(trigger);
		}

		[Fact]
		public void Remove_WithTrigger_RemovesTrigger() {
			var config = new TriggerConfig();
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();
			var trigger = new Trigger(bus, string.Empty, TriggerType.All, Enumerable.Empty<TriggerAction>(), Enumerable.Empty<Condition>(), logProvider);
			config.Add(trigger);
			config.Remove(trigger.Id);
			config.Triggers.Should().BeEmpty();
		}

		[Fact(Skip = "Too fragile")]
		public void Save_WithFilename_SavesConfigToJson() {
			var config = new TriggerConfig();
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var device = new DeviceStub(bus, "topic", "device");
			var group=new DeviceGroup("group");
			var actions = new[] {
				new TriggerAction(TriggerActionType.Device, device.Topic, device.Method),
				new TriggerAction(TriggerActionType.Group, group.Name, group.TurnOn)
			};
			var conditions = new[] {
				new Condition(Comparator.Equal, "event", "value"),
				new DeviceCondition(Comparator.Equal, "event", device)
			};
			var trigger = new Trigger(bus, "name", TriggerType.Any, actions, conditions, logProvider) {
				Enabled = false
			};
			config.Add(trigger);

			var testJsonPath = Path.Combine(Useful.GetTestDirectory(), "test.json");
			config.Save(testJsonPath);
			File.ReadAllText(testJsonPath).Should().BeEquivalentTo(ExpectedJson);
			File.Delete(testJsonPath);
		}

		[Fact]
		public void Load_ReturnsValidTriggerConfig() {
			var testJsonPath = Path.Combine(Useful.GetTestDirectory(), "test.json");
			File.WriteAllText(testJsonPath, ExpectedJson);
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();
			var baseDevices = new Dictionary<string, BaseDevice> {{"topic", new DeviceStub(bus, "topic", "device")}};
			var groups = new Dictionary<string, string[]> {{"group", new[] {"topic"}}};
			var deviceConfig = new DeviceConfig(baseDevices, groups);
			var config = TriggerConfig.Load(testJsonPath, bus, deviceConfig, logProvider);

			var trigger = config.Triggers.FirstOrDefault(t => t.Name=="name");
			trigger.Should().NotBeNull();
			trigger.Conditions.First().Should().BeOfType<Condition>();
			trigger.Conditions.Last().Should().BeOfType<DeviceCondition>();
			trigger.Actions.First().Identifier.Should().Be("topic");
			trigger.Actions.First().Type.Should().Be(TriggerActionType.Device);
			trigger.Actions.Last().Identifier.Should().Be("group");
			trigger.Actions.Last().Type.Should().Be(TriggerActionType.Group);
			trigger.Enabled.Should().BeFalse();
			File.Delete(testJsonPath);
		}
	}
}
