﻿using System.Linq;
using FluentAssertions;
using HAServer.Core;
using HAServer.Interfaces.Core;
using NSubstitute;
using Xunit;

namespace HAServer.Tests.Core {
	public class TriggerTests {
		[Fact]
		public void Constructor_WithAllParameters_PropertiesAreCorrectlySetUp() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();
			var condition = new Condition(Comparator.Equal, "test", "");
			var conditions = new[] { condition };
			var action = new TriggerAction(TriggerActionType.Device, "device", () => { });
			var actions = new[] { action };
			var trigger = new Trigger(bus, "name", TriggerType.All, actions, conditions, logProvider);

			trigger.Name.Should().Be("name");
			trigger.TriggerType.Should().Be(TriggerType.All);
			trigger.Conditions.Should().Contain(condition);
			trigger.Actions.Should().Contain(action);
			trigger.Enabled.Should().BeTrue();
		}

		[Fact]
		public void AddCondition_WithConditionsWithTheSameTopic_HasUniqueEntries() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => { }) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "test", "on"),
				new Condition(Comparator.NotEqual, "test", "off")
			};
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.Conditions.Should().HaveCount(2);
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithOneConditionMet_ExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] { new Condition(Comparator.Equal, "test", "on") };
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "on");

			triggered.Should().BeTrue();
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithNoConditionsMet_DoNotExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] { new Condition(Comparator.Equal, "test", "on") };
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "off");

			triggered.Should().BeFalse();
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithMutipleConditionsAndOneIsMet_ExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "test", "on"),
				new Condition(Comparator.Equal, "test2", "on")
			};
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "off");
			trigger.ProcessMessage("test2", "on");

			triggered.Should().BeTrue();
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithMutipleConditionsAndNoneAreMet_DoNotExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "test", "on"),
				new Condition(Comparator.Equal, "test2", "on")
			};
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "off");
			trigger.ProcessMessage("test2", "off");

			triggered.Should().BeFalse();
		}

		[Fact]
		public void WhenAllConditionsAreTrue_WithOneConditionMet_DoNotExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "test", "on"),
				new Condition(Comparator.Equal, "test2", "on")
			};
			var trigger = new Trigger(bus, "", TriggerType.All, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "on");
			trigger.ProcessMessage("test2", "off");

			triggered.Should().BeFalse();
		}

		[Fact]
		public void WhenAllConditionsAreTrue_WithNoConditionsMet_DoNotExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "test", "on"),
				new Condition(Comparator.Equal, "test2", "on")
			};
			var trigger = new Trigger(bus, "", TriggerType.All, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "off");
			trigger.ProcessMessage("test2", "off");

			triggered.Should().BeFalse();
		}

		[Fact]
		public void WhenAllConditionsAreTrue_WithAllConditionsMet_ExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "test", "on"),
				new Condition(Comparator.Equal, "test2", "on")
			};
			var trigger = new Trigger(bus, "", TriggerType.All, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "on");
			trigger.ProcessMessage("test2", "on");

			triggered.Should().BeTrue();
		}

		[Fact]
		public void WhenAllConditionsAreTrue_WithAllConditionsMet_ExecuteAction2() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.Equal, "solar_event", "civil_twilight_end"),
				new Condition(Comparator.Equal, "time", "21:00")
			};
			var trigger = new Trigger(bus, "", TriggerType.All, actions, conditions, logProvider);

			trigger.ProcessMessage("solar_event", "civil_twilight_end");
			trigger.ProcessMessage("time", "21:00:00");

			triggered.Should().BeTrue();
		}

		[Fact]
		public void WhenAllConditionsAreTrue_WithAllConditionsMet_ExecuteAction3() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {
				new Condition(Comparator.GreaterThan, "test", "1"),
				new Condition(Comparator.LessThan, "test", "3")
			};
			var trigger = new Trigger(bus, "", TriggerType.All, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "2");

			triggered.Should().BeTrue();
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithConditionMetMultipleTimes_ExecuteActionOnlyOnce() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var count = 0;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => count++) };
			var conditions = new[] { new Condition(Comparator.GreaterThan, "test", "0") };
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "1");
			trigger.ProcessMessage("test", "2");

			count.Should().Be(1);
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithConditionIsToggled_ExecuteActionCorrectAmountOfTimes() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var count = 0;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => count++) };
			var conditions = new[] { new Condition(Comparator.GreaterThan, "test", "0") };
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "1");
			trigger.ProcessMessage("test", "0");
			trigger.ProcessMessage("test", "2");

			count.Should().Be(2);
		}

		[Fact]
		public void WhenTriggerIsDisabledAndAnyConditionIsTrue_WithOneConditionMet_ExecuteAction() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var triggered = false;
			var actions = new[] { new TriggerAction(TriggerActionType.Device, "device", () => triggered = true) };
			var conditions = new[] {new Condition(Comparator.Equal, "test", "on")};
			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider) {
				Enabled = false
			};

			trigger.ProcessMessage("test", "on");

			triggered.Should().BeFalse();
		}

		[Fact]
		public void WhenAnyConditionIsTrue_WithOneConditionMet_ExecuteAllActions() {
			var bus = Substitute.For<IMessageBus>();
			var logProvider = Substitute.For<ILogProvider>();

			var deviceTriggered = false;
			var groupTriggered = false;
			var actions = new[] {
				new TriggerAction(TriggerActionType.Device, "device", ()=>deviceTriggered=true),
				new TriggerAction(TriggerActionType.Group, "group", ()=>groupTriggered=true)
			};
			var conditions = new[] {new Condition(Comparator.Equal, "test", "on")};

			var trigger = new Trigger(bus, "", TriggerType.Any, actions, conditions, logProvider);

			trigger.ProcessMessage("test", "on");

			deviceTriggered.Should().BeTrue();
			groupTriggered.Should().BeTrue();
		}
	}
}
