﻿using FluentAssertions;
using HAServer.Core;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using NSubstitute;
using Xunit;

namespace HAServer.Tests.Devices {
	public class SwitchTests {
		[Fact]
		public void WhenGivenOnSignal_OnIsTrue() {
			var log = Substitute.For<ILogProvider>();
			var bus = new MessageBus(log);
			var device=new Switch(bus, "test", "test", "1", "0");
			bus.PublishMessage("test", "1", null);
			device.On.Should().BeTrue();
		}

		[Fact]
		public void WhenGivenOffSignal_OnIsFalse() {
			var log = Substitute.For<ILogProvider>();
			var bus = new MessageBus(log);
			var device = new Switch(bus, "test", "test", "1", "0") {On = true};
			bus.PublishMessage("test", "0", null);
			device.On.Should().BeFalse();
		}

		[Fact]
		public void WhenOnIsSetToFalse_ValueIsEqualToOffValue() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0");

			bus.When(messageBus => messageBus.PublishMessage("test", Arg.Any<string>(), Arg.Any<ISubscriber>()))
			   .Do(callInfo => {
				   device.ProcessMessage(callInfo.ArgAt<string>(0), callInfo.ArgAt<string>(1));
			   });

			device.On = false;
			Assert.Equal("0", device.OffValue);
			Assert.Equal(device.OffValue, device.Value);
		}

		[Fact]
		public void WhenOnIsSetToTrue_ValueIsEqualToOnValue() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0");

			bus.When(messageBus => messageBus.PublishMessage("test", Arg.Any<string>(), Arg.Any<ISubscriber>()))
			   .Do(callInfo => {
				   device.ProcessMessage(callInfo.ArgAt<string>(0), callInfo.ArgAt<string>(1));
			   });

			device.On = true;
			Assert.Equal("1", device.OnValue);
			Assert.Equal(device.OnValue, device.Value);
		}

		[Fact]
		public void WhenOnIsSetToFalse_OffValueIsPublished() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0");
			var callbackMessage = string.Empty;

			bus.When(messageBus => messageBus.PublishMessage("test", Arg.Any<string>(), Arg.Any<ISubscriber>()))
			   .Do(callInfo => { callbackMessage = callInfo.ArgAt<string>(1); });

			device.On = false;
			Assert.Equal(device.OffValue, callbackMessage);
		}

		[Fact]
		public void WhenOnIsSetToTrue_OnValueIsPublished() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0");
			var callbackMessage = string.Empty;

			bus.When(messageBus => messageBus.PublishMessage("test", Arg.Any<string>(), Arg.Any<ISubscriber>()))
			   .Do(callInfo => { callbackMessage = callInfo.ArgAt<string>(1); });

			device.On = true;
			Assert.Equal(device.OnValue, callbackMessage);
		}

		[Fact]
		public void WhenOnIsSetToTrue_DeviceUpdateIsPublished() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0");
			var callbackMessage = string.Empty;

			bus.When(messageBus => messageBus.PublishMessage(Switch.DeviceUpdateTopic, Arg.Any<string>(), Arg.Any<ISubscriber>()))
			   .Do(callInfo => {
				   callbackMessage = callInfo.ArgAt<string>(1);
			   });

			device.On = true;
			Assert.Equal(device.Topic, callbackMessage);
		}

		[Fact]
		public void TurnOn_OnIsTrue() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0") {On = false};
			device.TurnOn();
			device.On.Should().BeTrue();
		}

		[Fact]
		public void TurnOff_OnIsFlase() {
			var bus = Substitute.For<IMessageBus>();
			var device = new Switch(bus, "test", "test", "1", "0") { On = true };
			device.TurnOff();
			device.On.Should().BeFalse();
		}
	}
}
