﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using HAServer.Serialisation;
using Newtonsoft.Json;

namespace HAServer.Core {
	public class DeviceConfig {
		private readonly Dictionary<string, BaseDevice> _devices = new Dictionary<string, BaseDevice>();
		private readonly DeviceGroupCollection _groups=new DeviceGroupCollection();
		private string _filename = "devices.json";

		public static DeviceConfig Load(string filename, IMessageBus messageBus) {
			var settings = new JsonSerialiserSettings(messageBus, null, null);
			var readAllText = File.ReadAllText(filename);
			var deviceConfig = JsonConvert.DeserializeObject<DeviceConfig>(readAllText, settings);
			deviceConfig._filename = filename;
			return deviceConfig;
		}

		public DeviceConfig() {
			
		}

		[JsonConstructor]
		public DeviceConfig(Dictionary<string, BaseDevice> devices, Dictionary<string, string[]> groups) {
			_devices = devices;
			foreach (var group in groups) {
				var deviceGroup = new DeviceGroup(group.Key);
				foreach (var groupTopic in group.Value) {
					deviceGroup.Add(_devices.First(pair => pair.Value.Topic == groupTopic).Value);
				}
				_groups.Add(deviceGroup);
			}
		}

		public void AddDevice(BaseDevice device) {
			_devices[device.Topic] = device;
		}

		public void RemoveDevice(string topic) {
			var device = _devices[topic];
			foreach (var group in _groups) {
				group.Remove(device);
			}
			_devices.Remove(topic);
		}

		public void AddGroup(string groupName) {
			if (_groups.All(group => group.Name != groupName))
				_groups.Add(new DeviceGroup(groupName));
		}

		public void RemoveGroup(string groupName) {
			var group = _groups.FirstOrDefault(g => g.Name == groupName);
			if (group!=null)
				_groups.Remove(group);
		}

		public void AddDeviceToGroup(string groupName, string topic) {
			if (!_devices.ContainsKey(topic))
				throw new ArgumentException("Device is not registered");

			var group = _groups.FirstOrDefault(g => g.Name == groupName);
			if (group==null)
				throw new ArgumentException("Group is not registered");

			group.Add(_devices[topic]);
		}

		public void RemoveDeviceFromGroup(string groupName, string deviceTopic) {
			_groups.FirstOrDefault(g => g.Name == groupName)?.Remove(_devices[deviceTopic]);
		}

		public void Save(string filename = null) {
			File.WriteAllText(filename ?? _filename, JsonConvert.SerializeObject(this, Formatting.Indented, new JsonSerialiserSettings(null, null, null)));
		}

		public IDictionary<string, BaseDevice> Devices => _devices;

		public DeviceGroupCollection Groups => _groups;
	}
}
