﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using HAServer.Interfaces.Core;

namespace HAServer.Core {
	public class Trigger:ISubscriber, IDisposable {
		private readonly IMessageBus _messageBus;
		private readonly List<Condition> _conditions = new List<Condition>();
		private readonly Dictionary<Condition, bool> _results = new Dictionary<Condition, bool>();
		private readonly ILogProvider _logProvider;

		public Trigger(IMessageBus messageBus, string name, TriggerType triggerType, IEnumerable<TriggerAction> actions, IEnumerable<Condition> conditions, ILogProvider logProvider) {
			_messageBus = messageBus;
			Actions = actions;
			_logProvider = logProvider;
			Name = name;
			TriggerType = triggerType;

			foreach (var condition in conditions) {
				AddCondition(condition);
			}

			Enabled = true;
		}

		private void AddCondition(Condition condition) {
			_results[condition] = false;
			_conditions.Add(condition);
			_messageBus.AddSubscriber(this, $"^{condition.Topic}$");
		}

		private bool Process(string topic, string value) {
			var conditions = _conditions.Where(condition => Regex.Match(topic, $"^{condition.Topic}$").Success);
			var haveAnyConditionsChanged = false;
			foreach (var condition in conditions) {
				var previousValue = _results[condition];
				_results[condition] = condition.IsTrue(value);
				if (_results[condition] != previousValue) {
					_logProvider.Write($"Trigger '{Name}': condition result for {topic} is now {_results[condition]}", LogLevel.Info);
					haveAnyConditionsChanged = true;
				}
			}

			if (!haveAnyConditionsChanged)
				return false;

			switch (TriggerType) {
				case TriggerType.Any:
					return _results.Any(result => result.Value);
				case TriggerType.All:
					return _results.All(result => result.Value);
				default:
					throw new ArgumentOutOfRangeException();
			}
		}

		public void ProcessMessage(string topic, string message) {
			if (!Enabled)
				return;

			var result = Process(topic, message);
			if (result) {
				foreach (var action in Actions)
					action.Execute();
				_logProvider.Write($"Trigger '{Name}' has been run", LogLevel.Info);
			}
		}

		public void Dispose() {
			_messageBus.RemoveSubscriber(this);
		}

		public int Id { get; set; }
		public string Name { get; }
		public TriggerType TriggerType { get; }
		public bool Enabled { get; set; }
		public IEnumerable<Condition> Conditions => _conditions;
		public IEnumerable<TriggerAction> Actions { get; }
	}
}
