﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Linq;
using HAServer.Core;
using HAServer.Interfaces.Core;
using HAServer.WebServer;

namespace HAServer {
	[ExcludeFromCodeCoverage]
	public static class Program {
		private static WebUI _webUi;
		private static MessageBus _messageBus;
		private static ComponentLoader<IComponent> _componentLoader;

		public static void Main(string[] args) {
			//if (!Environment.UserInteractive)
				Start();
			//else {
			//	Start();
			//	while (ReadCommand()) ;
			//	Stop();
			//}
		}

		private static bool ReadCommand()
		{
			var command = Console.ReadLine();
			var parts = command.Split(' ');
			switch (parts.First().ToLower())
			{
				case "quit":
					return false;
				case "publish":
					Publish(parts[1], parts[2]);
					break;
			}

			return true;
		}

		private static void Publish(string topic, string message)
		{
			_messageBus.PublishMessage(topic, message, null);
		}

		public static void Start() {
			var dependencyResolver = new DependencyResolver();
			dependencyResolver.Register<ILogProvider>(new Log("log.txt", LogLevel.Info, LogLevel.Debug));
			var settingsProvider = new ApplicationSettingsProvider();
			dependencyResolver.Register<ISettingsProvider>(settingsProvider);
			dependencyResolver.Register<IApplicationSettingsProvider>(settingsProvider);
			_messageBus = new MessageBus(dependencyResolver.Resolve<ILogProvider>());
			dependencyResolver.Register<IMessageBus>(_messageBus);
			dependencyResolver.Register<IFileGet>(new HttpGet());
			dependencyResolver.Register(TimeZoneInfo.Local);

			var devices = File.Exists("devices.json")
							  ? DeviceConfig.Load("devices.json", dependencyResolver.Resolve<IMessageBus>())
							  : new DeviceConfig();
			dependencyResolver.Register(devices);

			var triggers = File.Exists("triggers.json")
							   ? TriggerConfig.Load(
												    "triggers.json",
												    dependencyResolver.Resolve<IMessageBus>(),
												    devices,
												    dependencyResolver.Resolve<ILogProvider>())
							   : new TriggerConfig();
			dependencyResolver.Register(triggers);

			_componentLoader = new ComponentLoader<IComponent>("Components", dependencyResolver);
			_componentLoader.Load();
			foreach (var component in _componentLoader.Components) {
				component.Initialise();
			}

			_webUi = dependencyResolver.CreateInstance<WebUI>();
			_webUi.Start();
		}

		public static void Stop() {
			_webUi.Stop();
			foreach (var component in _componentLoader.Components) {
				component.Shutdown();
			}
		}

	}
}
