﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using HAServer.Core;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using HAServer.Serialisation;
using HAServer.WebServer.DTO;
using HAServer.WebServer.RequestHandlers;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using WebSocketSharp;
using WebSocketSharp.Server;
using LogLevel = HAServer.Interfaces.Core.LogLevel;

namespace HAServer.WebServer {
	public class WebSocketApi:WebSocketBehavior, ISubscriber {
		private const string UpdateDevicesTopic = "UpdateDevices";
		private const string UpdateGroupsTopic = "UpdateGroups";

		private readonly IMessageBus _messageBus;
		private readonly DeviceConfig _deviceConfig;
		private readonly TriggerConfig _triggerConfig;
		private readonly IApplicationSettingsProvider _settingsProvider;
		private readonly ILogProvider _logProvider;
		private readonly Dictionary<string, Action<string>> _topicActions;
		private readonly ApiMethods _apiMethods;
		private readonly JsonSerializer _serialiser;

		public WebSocketApi(IMessageBus messageBus, DeviceConfig deviceConfig, TriggerConfig triggerConfig, IApplicationSettingsProvider settingsProvider, ILogProvider logProvider, WebObjectHandler webObjectHandler) {
			_messageBus = messageBus;
			_deviceConfig = deviceConfig;
			_triggerConfig = triggerConfig;
			_settingsProvider = settingsProvider;
			_logProvider = logProvider;

			_topicActions=new Dictionary<string, Action<string>> {
				{BaseDevice.DeviceUpdateTopic, UpdateDevice},
				{UpdateDevicesTopic, UpdateDevices},
				{UpdateGroupsTopic, UpdateGroups}
			};

			_apiMethods=new ApiMethods(this, _deviceConfig, _triggerConfig, settingsProvider, webObjectHandler);

			var settings = new JsonSerialiserSettings(messageBus, deviceConfig, logProvider);
			_serialiser = JsonSerializer.Create(settings);
		}

		private void SendResult(RemoteProcedureCall result) {
			TextWriter writer=new StringWriter();
			_serialiser.Serialize(writer, result);
			var data = writer.ToString();
			Send(data);
		}

		private void UpdateDevice(string deviceTopic) {
			if (!_deviceConfig.Devices.ContainsKey(deviceTopic))
				return;

			var result=new RemoteProcedureCall("UpdateDevice", new DeviceDto(_deviceConfig.Devices[deviceTopic]));
			SendResult(result);
		}

		public void UpdateDevices(bool publish) {
			var result=new RemoteProcedureCall(UpdateDevicesTopic, _deviceConfig.Devices.Select(pair => new DeviceDto(pair.Value)));
			SendResult(result);
			if (publish)
				_messageBus.PublishMessage(UpdateDevicesTopic, "", this);
		}

		private void UpdateDevices(string obj) {
			UpdateDevices(false);
		}

		public void UpdateGroups(bool publish) {
			var result=new RemoteProcedureCall(UpdateGroupsTopic, _deviceConfig.Groups);
			SendResult(result);
			if (publish)
				_messageBus.PublishMessage(UpdateGroupsTopic, "", this);
		}

		private void UpdateGroups(string obj) {
			UpdateGroups(false);
		}

		public void UpdateSettings() {
			var result=new RemoteProcedureCall("UpdateSettings", new SettingsDto(_settingsProvider));
			SendResult(result);
		}

		public void UpdateTriggers() {
			var result=new RemoteProcedureCall("UpdateTriggers", _triggerConfig.Triggers.Select(trigger => new TriggerDto(trigger)));
			SendResult(result);
		}

		private void SendInitialData() {
			UpdateDevices(false);
			UpdateGroups(false);
			UpdateTriggers();
			UpdateSettings();
		}

		protected override void OnClose(CloseEventArgs e) {
			_messageBus.RemoveSubscriber(this);
		}

		protected override void OnOpen() {
			foreach (var topicAction in _topicActions) {
				_messageBus.AddSubscriber(this, topicAction.Key);
			}
			SendInitialData();
		}

		protected override void OnMessage(MessageEventArgs e) {
			try {
				var rpc = JsonConvert.DeserializeObject<RemoteProcedureCall>(e.Data);
				var parameters=new object[rpc.Parameters.Length];
				var method = _apiMethods.GetType().GetMethod(rpc.Method);
				var methodParameters = method.GetParameters();

				for (var i = 0; i < parameters.Length; i++) {
					if (rpc.Parameters[i].GetType() == typeof(JObject)) {
						var jObject = (JObject)rpc.Parameters[i];
						parameters[i] = jObject.ToObject(methodParameters[i].ParameterType, _serialiser);
					} else {
						parameters[i] = rpc.Parameters[i];
					}
				}

				method.Invoke(_apiMethods, parameters);
			} catch (Exception exception) {
				_logProvider.Write(exception.ToString(), LogLevel.Error);
				throw;
			}
		}

		public void ProcessMessage(string topic, string message) {
			if (_topicActions.ContainsKey(topic))
				_topicActions[topic](message);
		}
	}
}
