﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using HAServer.Core;
using HAServer.Interfaces.Core;
using HAServer.WebServer.RequestHandlers;
using WebSocketSharp.Server;

namespace HAServer.WebServer {
	[ExcludeFromCodeCoverage]
	public class WebUI {
		private readonly HttpServer _server;
		private readonly IMessageBus _messageBus;
		private readonly DeviceConfig _deviceConfig;
		private readonly TriggerConfig _triggerConfig;
		private readonly IApplicationSettingsProvider _settingsProvider;
		private readonly ILogProvider _logProvider;
		private readonly Dictionary<string, IWebRequestHandler> _requestHandlers;
		private readonly WebObjectHandler _webObjectHandler;

		public WebUI(IMessageBus messageBus, DeviceConfig deviceConfig, TriggerConfig triggerConfig, IApplicationSettingsProvider settingsProvider, ILogProvider logProvider) {
			_messageBus = messageBus;
			_deviceConfig = deviceConfig;
			_triggerConfig = triggerConfig;
			_settingsProvider = settingsProvider;
			_logProvider = logProvider;
			_server = new HttpServer(settingsProvider.WebUiPort);
			_server.OnGet += GetHandler;
			_server.AddWebSocketService("/api", GetNewWebsocketApi);

			_webObjectHandler = new WebObjectHandler(settingsProvider.WebUiPath);
			_requestHandlers=new Dictionary<string, IWebRequestHandler> {
				{$"^/{WebObjectHandler.ObjectFolder}/.*", _webObjectHandler},
				{".*", new DefaultWebRequestHandler(settingsProvider.WebUiPath)}
			};
		}

		private WebSocketApi GetNewWebsocketApi() {
			return new WebSocketApi(_messageBus,_deviceConfig,_triggerConfig, _settingsProvider, _logProvider, _webObjectHandler);
		}

		void GetHandler(object sender, HttpRequestEventArgs e) {
			var path = e.Request.Url.AbsolutePath;
			byte[] response = { };

			try {
				var handler = _requestHandlers.First(pair => Regex.IsMatch(path, pair.Key)).Value;
				response = handler.GetContent(e);
			} catch (FileNotFoundException exception) {
				_logProvider.Write(exception.Message, LogLevel.Warning);
				e.Response.StatusCode = 404;
			} catch (Exception exception) {
				_logProvider.Write(exception.Message, LogLevel.Error);
				e.Response.StatusCode = 500;
			}

			e.Response.ContentEncoding = Encoding.UTF8;
			e.Response.ContentLength64 = response.Length;
			e.Response.OutputStream.Write(response, 0, response.Length);
		}

		public void Start() {
			_server.Start();
		}

		public void Stop() {
			_server.Stop();
		}
	}
}
